/**
 * Copyright (c) 2009 Justin Aquadro
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef FONTMANAGER_H_
#define FONTMANAGER_H_

#include <SFML/Graphics.hpp>
#include <map>
#include <string>
#include <stdexcept>

class FontManager {
public:

	class FontMissingException : public std::runtime_error {
	public:

		FontMissingException ()
			: std::runtime_error("Missing Font Exception") {
		}

		FontMissingException (const std::string& str)
			: std::runtime_error("Missing Font Exception: " + str) {
		}
	};

protected:

	std::map<std::string, sf::Font> _fonts;

public:

	void addFont (const std::string& fontName, const std::string& fontPath, int size) {
		_fonts[fontName];

		if (!_fonts[fontName].LoadFromFile(fontPath, size)) {
			throw FontMissingException("Could not find font at '" + fontPath + "'");
		}
	}

	const sf::Font& getFont (const std::string& fontName) {
		std::map<std::string, sf::Font>::const_iterator font = _fonts.find(fontName);
		if (font == _fonts.end()) {
			throw FontMissingException("No font record for '" + fontName + "'");
		}

		return font->second;
	}

	static FontManager& getManager () {
		static FontManager fm;
		return fm;
	}

protected:

	FontManager () { }

	FontManager (const FontManager&) { }

	FontManager& operator= (const FontManager&) {
		return *this;
	}

};

#endif /* FONTMANAGER_H_ */
